@@ -12,15 +12,13 @@ const uint32_t COOLIX_LED = 0xB5F5A5;
 const uint32_t COOLIX_SILENCE_FP = 0xB5F5B6;
 
 // On, 25C, Mode: Auto, Fan: Auto, Zone Follow: Off, Sensor Temp: Ignore.
-const uint32_t COOLIX_DEFAULT_STATE = 0xB2BFC8;
-const uint32_t COOLIX_DEFAULT_STATE_AUTO_24_FAN = 0xB21F48;
 const uint8_t COOLIX_COOL = 0b0000;
 const uint8_t COOLIX_DRY_FAN = 0b0100;
 const uint8_t COOLIX_AUTO = 0b1000;
 const uint8_t COOLIX_HEAT = 0b1100;
 const uint32_t COOLIX_MODE_MASK = 0b1100;
 const uint32_t COOLIX_FAN_MASK = 0xF000;
-const uint32_t COOLIX_FAN_DRY = 0x1000;
+const uint32_t COOLIX_FAN_MODE_AUTO_DRY = 0x1000;
 const uint32_t COOLIX_FAN_AUTO = 0xB000;
 const uint32_t COOLIX_FAN_MIN = 0x9000;
 const uint32_t COOLIX_FAN_MED = 0x5000;
@@ -28,23 +26,23 @@ const uint32_t COOLIX_FAN_MAX = 0x3000;
 
 // Temperature
 const uint8_t COOLIX_TEMP_RANGE = COOLIX_TEMP_MAX - COOLIX_TEMP_MIN + 1;
-const uint8_t COOLIX_FAN_TEMP_CODE = 0b1110;  // Part of Fan Mode.
+const uint8_t COOLIX_FAN_TEMP_CODE = 0b11100000;  // Part of Fan Mode.
 const uint32_t COOLIX_TEMP_MASK = 0b11110000;
 const uint8_t COOLIX_TEMP_MAP[COOLIX_TEMP_RANGE] = {
-    0b0000,  // 17C
-    0b0001,  // 18c
-    0b0011,  // 19C
-    0b0010,  // 20C
-    0b0110,  // 21C
-    0b0111,  // 22C
-    0b0101,  // 23C
-    0b0100,  // 24C
-    0b1100,  // 25C
-    0b1101,  // 26C
-    0b1001,  // 27C
-    0b1000,  // 28C
-    0b1010,  // 29C
-    0b1011   // 30C
+    0b00000000,  // 17C
+    0b00010000,  // 18c
+    0b00110000,  // 19C
+    0b00100000,  // 20C
+    0b01100000,  // 21C
+    0b01110000,  // 22C
+    0b01010000,  // 23C
+    0b01000000,  // 24C
+    0b11000000,  // 25C
+    0b11010000,  // 26C
+    0b10010000,  // 27C
+    0b10000000,  // 28C
+    0b10100000,  // 29C
+    0b10110000   // 30C
 };
 
 // Constants
@@ -59,29 +57,60 @@ static const uint32_t FOOTER_SPACE_US = HEADER_SPACE_US;
 const uint16_t COOLIX_BITS = 24;
 
 void CoolixClimate::transmit_state() {
-  uint32_t remote_state;
+  uint32_t remote_state = 0xB20F00;
 
-  switch (this->mode) {
-    case climate::CLIMATE_MODE_COOL:
-      remote_state = (COOLIX_DEFAULT_STATE & ~COOLIX_MODE_MASK) | COOLIX_COOL;
-      break;
-    case climate::CLIMATE_MODE_HEAT:
-      remote_state = (COOLIX_DEFAULT_STATE & ~COOLIX_MODE_MASK) | COOLIX_HEAT;
-      break;
-    case climate::CLIMATE_MODE_AUTO:
-      remote_state = COOLIX_DEFAULT_STATE_AUTO_24_FAN;
-      break;
-    case climate::CLIMATE_MODE_OFF:
-    default:
-      remote_state = COOLIX_OFF;
-      break;
+  if (send_swing_cmd_) {
+    send_swing_cmd_ = false;
+    remote_state = COOLIX_SWING;
+  } else {
+    switch (this->mode) {
+      case climate::CLIMATE_MODE_COOL:
+        remote_state |= COOLIX_COOL;
+        break;
+      case climate::CLIMATE_MODE_HEAT:
+        remote_state |= COOLIX_HEAT;
+        break;
+      case climate::CLIMATE_MODE_HEAT_COOL:
+        remote_state |= COOLIX_AUTO;
+        break;
+      case climate::CLIMATE_MODE_FAN_ONLY:
+      case climate::CLIMATE_MODE_DRY:
+        remote_state |= COOLIX_DRY_FAN;
+        break;
+      case climate::CLIMATE_MODE_OFF:
+      default:
+        remote_state = COOLIX_OFF;
+        break;
+    }
+    if (this->mode != climate::CLIMATE_MODE_OFF) {
+      if (this->mode != climate::CLIMATE_MODE_FAN_ONLY) {
+        auto temp = (uint8_t) roundf(clamp(this->target_temperature, COOLIX_TEMP_MIN, COOLIX_TEMP_MAX));
+        remote_state |= COOLIX_TEMP_MAP[temp - COOLIX_TEMP_MIN];
+      } else {
+        remote_state |= COOLIX_FAN_TEMP_CODE;
+      }
+      if (this->mode == climate::CLIMATE_MODE_HEAT_COOL || this->mode == climate::CLIMATE_MODE_DRY) {
+        this->fan_mode = climate::CLIMATE_FAN_AUTO;
+        remote_state |= COOLIX_FAN_MODE_AUTO_DRY;
+      } else {
+        switch (this->fan_mode.value()) {
+          case climate::CLIMATE_FAN_HIGH:
+            remote_state |= COOLIX_FAN_MAX;
+            break;
+          case climate::CLIMATE_FAN_MEDIUM:
+            remote_state |= COOLIX_FAN_MED;
+            break;
+          case climate::CLIMATE_FAN_LOW:
+            remote_state |= COOLIX_FAN_MIN;
+            break;
+          case climate::CLIMATE_FAN_AUTO:
+          default:
+            remote_state |= COOLIX_FAN_AUTO;
+            break;
+        }
+      }
+    }
   }
-  if (this->mode != climate::CLIMATE_MODE_OFF) {
-    auto temp = (uint8_t) roundf(clamp(this->target_temperature, COOLIX_TEMP_MIN, COOLIX_TEMP_MAX));
-    remote_state &= ~COOLIX_TEMP_MASK;  // Clear the old temp.
-    remote_state |= COOLIX_TEMP_MAP[temp - COOLIX_TEMP_MIN] << 4;
-  }
-
   ESP_LOGV(TAG, "Sending coolix code: 0x%02X", remote_state);
 
   auto transmit = this->transmitter_->transmit();
@@ -161,35 +190,35 @@ bool CoolixClimate::on_receive(remote_base::RemoteReceiveData data) {
 
   if (remote_state == COOLIX_OFF) {
     this->mode = climate::CLIMATE_MODE_OFF;
+  } else if (remote_state == COOLIX_SWING) {
+    this->swing_mode =
+        this->swing_mode == climate::CLIMATE_SWING_OFF ? climate::CLIMATE_SWING_VERTICAL : climate::CLIMATE_SWING_OFF;
   } else {
     if ((remote_state & COOLIX_MODE_MASK) == COOLIX_HEAT)
       this->mode = climate::CLIMATE_MODE_HEAT;
     else if ((remote_state & COOLIX_MODE_MASK) == COOLIX_AUTO)
-      this->mode = climate::CLIMATE_MODE_AUTO;
+      this->mode = climate::CLIMATE_MODE_HEAT_COOL;
     else if ((remote_state & COOLIX_MODE_MASK) == COOLIX_DRY_FAN) {
-      // climate::CLIMATE_MODE_DRY;
-      if ((remote_state & COOLIX_FAN_MASK) == COOLIX_FAN_DRY)
-        ESP_LOGV(TAG, "Not supported DRY mode. Reporting AUTO");
+      if ((remote_state & COOLIX_FAN_MASK) == COOLIX_FAN_MODE_AUTO_DRY)
+        this->mode = climate::CLIMATE_MODE_DRY;
       else
-        ESP_LOGV(TAG, "Not supported FAN Auto mode. Reporting AUTO");
-      this->mode = climate::CLIMATE_MODE_AUTO;
+        this->mode = climate::CLIMATE_MODE_FAN_ONLY;
     } else
       this->mode = climate::CLIMATE_MODE_COOL;
 
     // Fan Speed
-    // When climate::CLIMATE_MODE_DRY is implemented replace following line with this:
-    //   if ((remote_state & COOLIX_FAN_AUTO) == COOLIX_FAN_AUTO || this->mode == climate::CLIMATE_MODE_DRY)
-    if ((remote_state & COOLIX_FAN_AUTO) == COOLIX_FAN_AUTO)
-      ESP_LOGV(TAG, "Not supported FAN speed AUTO");
+    if ((remote_state & COOLIX_FAN_AUTO) == COOLIX_FAN_AUTO || this->mode == climate::CLIMATE_MODE_HEAT_COOL ||
+        this->mode == climate::CLIMATE_MODE_DRY)
+      this->fan_mode = climate::CLIMATE_FAN_AUTO;
     else if ((remote_state & COOLIX_FAN_MIN) == COOLIX_FAN_MIN)
-      ESP_LOGV(TAG, "Not supported FAN speed MIN");
+      this->fan_mode = climate::CLIMATE_FAN_LOW;
     else if ((remote_state & COOLIX_FAN_MED) == COOLIX_FAN_MED)
-      ESP_LOGV(TAG, "Not supported FAN speed MED");
+      this->fan_mode = climate::CLIMATE_FAN_MEDIUM;
     else if ((remote_state & COOLIX_FAN_MAX) == COOLIX_FAN_MAX)
-      ESP_LOGV(TAG, "Not supported FAN speed MAX");
+      this->fan_mode = climate::CLIMATE_FAN_HIGH;
 
     // Temperature
-    uint8_t temperature_code = (remote_state & COOLIX_TEMP_MASK) >> 4;
+    uint8_t temperature_code = remote_state & COOLIX_TEMP_MASK;
     for (uint8_t i = 0; i < COOLIX_TEMP_RANGE; i++)
       if (COOLIX_TEMP_MAP[i] == temperature_code)
         this->target_temperature = i + COOLIX_TEMP_MIN;
