@@ -1,4 +1,5 @@
 #include "speed_fan.h"
+#include "esphome/components/fan/fan_helpers.h"
 #include "esphome/core/log.h"
 
 namespace esphome {
@@ -11,9 +12,12 @@ void SpeedFan::dump_config() {
   if (this->fan_->get_traits().supports_oscillation()) {
     ESP_LOGCONFIG(TAG, "  Oscillation: YES");
   }
+  if (this->fan_->get_traits().supports_direction()) {
+    ESP_LOGCONFIG(TAG, "  Direction: YES");
+  }
 }
 void SpeedFan::setup() {
-  auto traits = fan::FanTraits(this->oscillating_ != nullptr, true);
+  auto traits = fan::FanTraits(this->oscillating_ != nullptr, true, this->direction_ != nullptr, this->speed_count_);
   this->fan_->set_traits(traits);
   this->fan_->add_on_state_callback([this]() { this->next_update_ = true; });
 }
@@ -26,12 +30,7 @@ void SpeedFan::loop() {
   {
     float speed = 0.0f;
     if (this->fan_->state) {
-      if (this->fan_->speed == fan::FAN_SPEED_LOW)
-        speed = this->low_speed_;
-      else if (this->fan_->speed == fan::FAN_SPEED_MEDIUM)
-        speed = this->medium_speed_;
-      else if (this->fan_->speed == fan::FAN_SPEED_HIGH)
-        speed = this->high_speed_;
+      speed = static_cast<float>(this->fan_->speed) / static_cast<float>(this->speed_count_);
     }
     ESP_LOGD(TAG, "Setting speed: %.2f", speed);
     this->output_->set_level(speed);
@@ -46,6 +45,16 @@ void SpeedFan::loop() {
     }
     ESP_LOGD(TAG, "Setting oscillation: %s", ONOFF(enable));
   }
+
+  if (this->direction_ != nullptr) {
+    bool enable = this->fan_->direction == fan::FAN_DIRECTION_REVERSE;
+    if (enable) {
+      this->direction_->turn_on();
+    } else {
+      this->direction_->turn_off();
+    }
+    ESP_LOGD(TAG, "Setting reverse direction: %s", ONOFF(enable));
+  }
 }
 float SpeedFan::get_setup_priority() const { return setup_priority::DATA; }
 
