@@ -1,8 +1,8 @@
 """Constants used by esphome."""
 
 MAJOR_VERSION = 1
-MINOR_VERSION = 18
-PATCH_VERSION = "0"
+MINOR_VERSION = 19
+PATCH_VERSION = "1"
 __short_version__ = f"{MAJOR_VERSION}.{MINOR_VERSION}"
 __version__ = f"{__short_version__}.{PATCH_VERSION}"
 
@@ -55,17 +55,20 @@ CONF_ACCELERATION_Z = "acceleration_z"
 CONF_ACCURACY = "accuracy"
 CONF_ACCURACY_DECIMALS = "accuracy_decimals"
 CONF_ACTION_ID = "action_id"
+CONF_ACTIVE_POWER = "active_power"
 CONF_ADDRESS = "address"
 CONF_ADDRESSABLE_LIGHT_ID = "addressable_light_id"
 CONF_ALPHA = "alpha"
 CONF_ALTITUDE = "altitude"
 CONF_AND = "and"
 CONF_AP = "ap"
+CONF_APPARENT_POWER = "apparent_power"
 CONF_ARDUINO_VERSION = "arduino_version"
 CONF_ARGS = "args"
 CONF_ASSUMED_STATE = "assumed_state"
 CONF_AT = "at"
 CONF_ATTENUATION = "attenuation"
+CONF_ATTRIBUTE = "attribute"
 CONF_AUTH = "auth"
 CONF_AUTO_MODE = "auto_mode"
 CONF_AUTOMATION_ID = "automation_id"
@@ -142,6 +145,10 @@ CONF_CSS_URL = "css_url"
 CONF_CURRENT = "current"
 CONF_CURRENT_OPERATION = "current_operation"
 CONF_CURRENT_RESISTOR = "current_resistor"
+CONF_CUSTOM_FAN_MODE = "custom_fan_mode"
+CONF_CUSTOM_FAN_MODES = "custom_fan_modes"
+CONF_CUSTOM_PRESET = "custom_preset"
+CONF_CUSTOM_PRESETS = "custom_presets"
 CONF_DALLAS_ID = "dallas_id"
 CONF_DATA = "data"
 CONF_DATA_PIN = "data_pin"
@@ -193,6 +200,8 @@ CONF_ESPHOME = "esphome"
 CONF_ETHERNET = "ethernet"
 CONF_EVENT = "event"
 CONF_EXPIRE_AFTER = "expire_after"
+CONF_EXPORT_ACTIVE_ENERGY = "export_active_energy"
+CONF_EXPORT_REACTIVE_ENERGY = "export_reactive_energy"
 CONF_EXTERNAL_COMPONENTS = "external_components"
 CONF_EXTERNAL_VCC = "external_vcc"
 CONF_FALLING_EDGE = "falling_edge"
@@ -217,6 +226,7 @@ CONF_FILTERS = "filters"
 CONF_FINGER_ID = "finger_id"
 CONF_FINGERPRINT_COUNT = "fingerprint_count"
 CONF_FLASH_LENGTH = "flash_length"
+CONF_FLOW_CONTROL_PIN = "flow_control_pin"
 CONF_FOR = "for"
 CONF_FORCE_UPDATE = "force_update"
 CONF_FORMALDEHYDE = "formaldehyde"
@@ -260,6 +270,8 @@ CONF_IF = "if"
 CONF_IIR_FILTER = "iir_filter"
 CONF_ILLUMINANCE = "illuminance"
 CONF_IMPEDANCE = "impedance"
+CONF_IMPORT_ACTIVE_ENERGY = "import_active_energy"
+CONF_IMPORT_REACTIVE_ENERGY = "import_reactive_energy"
 CONF_INCLUDES = "includes"
 CONF_INDEX = "index"
 CONF_INDOOR = "indoor"
@@ -379,6 +391,7 @@ CONF_ON_RELEASE = "on_release"
 CONF_ON_SHUTDOWN = "on_shutdown"
 CONF_ON_STATE = "on_state"
 CONF_ON_TAG = "on_tag"
+CONF_ON_TAG_REMOVED = "on_tag_removed"
 CONF_ON_TIME = "on_time"
 CONF_ON_TIME_SYNC = "on_time_sync"
 CONF_ON_TURN_OFF = "on_turn_off"
@@ -411,6 +424,7 @@ CONF_PAYLOAD = "payload"
 CONF_PAYLOAD_AVAILABLE = "payload_available"
 CONF_PAYLOAD_NOT_AVAILABLE = "payload_not_available"
 CONF_PERIOD = "period"
+CONF_PHASE_ANGLE = "phase_angle"
 CONF_PHASE_BALANCER = "phase_balancer"
 CONF_PIN = "pin"
 CONF_PIN_A = "pin_a"
@@ -439,8 +453,13 @@ CONF_POWER_FACTOR = "power_factor"
 CONF_POWER_ON_VALUE = "power_on_value"
 CONF_POWER_SAVE_MODE = "power_save_mode"
 CONF_POWER_SUPPLY = "power_supply"
+CONF_PRESET = "preset"
+CONF_PRESET_BOOST = "preset_boost"
+CONF_PRESET_ECO = "preset_eco"
+CONF_PRESET_SLEEP = "preset_sleep"
 CONF_PRESSURE = "pressure"
 CONF_PRIORITY = "priority"
+CONF_PROJECT = "project"
 CONF_PROTOCOL = "protocol"
 CONF_PULL_MODE = "pull_mode"
 CONF_PULSE_LENGTH = "pulse_length"
@@ -453,6 +472,7 @@ CONF_RATE = "rate"
 CONF_RAW = "raw"
 CONF_RC_CODE_1 = "rc_code_1"
 CONF_RC_CODE_2 = "rc_code_2"
+CONF_REACTIVE_POWER = "reactive_power"
 CONF_REBOOT_TIMEOUT = "reboot_timeout"
 CONF_RECEIVE_TIMEOUT = "receive_timeout"
 CONF_RED = "red"
@@ -469,6 +489,7 @@ CONF_RESTORE_STATE = "restore_state"
 CONF_RESTORE_VALUE = "restore_value"
 CONF_RETAIN = "retain"
 CONF_REVERSE_ACTIVE_ENERGY = "reverse_active_energy"
+CONF_REVERSED = "reversed"
 CONF_RGB_ORDER = "rgb_order"
 CONF_RGBW = "rgbw"
 CONF_RISING_EDGE = "rising_edge"
@@ -526,6 +547,7 @@ CONF_SPIKE_REJECTION = "spike_rejection"
 CONF_SSID = "ssid"
 CONF_SSL_FINGERPRINTS = "ssl_fingerprints"
 CONF_STATE = "state"
+CONF_STATE_CLASS = "state_class"
 CONF_STATE_TOPIC = "state_topic"
 CONF_STATIC_IP = "static_ip"
 CONF_STATUS = "status"
@@ -595,6 +617,7 @@ CONF_UUID = "uuid"
 CONF_VALUE = "value"
 CONF_VARIABLES = "variables"
 CONF_VARIANT = "variant"
+CONF_VERSION = "version"
 CONF_VISUAL = "visual"
 CONF_VOLTAGE = "voltage"
 CONF_VOLTAGE_ATTENUATION = "voltage_attenuation"
@@ -679,6 +702,7 @@ ICON_WIFI = "mdi:wifi"
 UNIT_AMPERE = "A"
 UNIT_CELSIUS = "°C"
 UNIT_COUNTS_PER_CUBIC_METER = "#/m³"
+UNIT_CUBIC_METER = "m³"
 UNIT_DECIBEL = "dB"
 UNIT_DECIBEL_MILLIWATT = "dBm"
 UNIT_DEGREE_PER_SECOND = "°/s"
@@ -711,6 +735,7 @@ UNIT_STEPS = "steps"
 UNIT_VOLT = "V"
 UNIT_VOLT_AMPS = "VA"
 UNIT_VOLT_AMPS_REACTIVE = "VAR"
+UNIT_VOLT_AMPS_REACTIVE_HOURS = "VARh"
 UNIT_WATT = "W"
 UNIT_WATT_HOURS = "Wh"
 
@@ -742,6 +767,8 @@ DEVICE_CLASS_EMPTY = ""
 DEVICE_CLASS_BATTERY = "battery"
 DEVICE_CLASS_POWER = "power"
 # device classes of sensor component
+DEVICE_CLASS_CARBON_MONOXIDE = "carbon_monoxide"
+DEVICE_CLASS_CARBON_DIOXIDE = "carbon_dioxide"
 DEVICE_CLASS_CURRENT = "current"
 DEVICE_CLASS_ENERGY = "energy"
 DEVICE_CLASS_HUMIDITY = "humidity"
@@ -752,3 +779,9 @@ DEVICE_CLASS_POWER_FACTOR = "power_factor"
 DEVICE_CLASS_PRESSURE = "pressure"
 DEVICE_CLASS_TIMESTAMP = "timestamp"
 DEVICE_CLASS_VOLTAGE = "voltage"
+
+# state classes
+STATE_CLASS_NONE = ""
+
+# The state represents a measurement in present time
+STATE_CLASS_MEASUREMENT = "measurement"
