@@ -38,8 +38,9 @@ namespace esphome {
 class Application {
  public:
   void pre_setup(const std::string &name, const char *compilation_time, bool name_add_mac_suffix) {
+    this->name_add_mac_suffix_ = name_add_mac_suffix;
     if (name_add_mac_suffix) {
-      this->name_ = name + "_" + get_mac_address().substr(6);
+      this->name_ = name + "-" + get_mac_address().substr(6);
     } else {
       this->name_ = name;
     }
@@ -97,6 +98,8 @@ class Application {
   /// Get the name of this Application set by set_name().
   const std::string &get_name() const { return this->name_; }
 
+  bool is_name_add_mac_suffix_enabled() const { return this->name_add_mac_suffix_; }
+
   const std::string &get_compilation_time() const { return this->compilation_time_; }
 
   /** Set the target interval with which to run the loop() calls.
@@ -245,6 +248,7 @@ class Application {
 
   std::string name_;
   std::string compilation_time_;
+  bool name_add_mac_suffix_;
   uint32_t last_loop_{0};
   uint32_t loop_interval_{16};
   int dump_config_at_{-1};
